/**
 * Define a locale object.
 * @function defineLocale
 */
'use strict'

const evaljson = require('evaljson')

const inflection = require('inflection')
const stringcase = require('stringcase')

/** @lends defineLocale */
function defineLocale (src) {
  /**
   * Locale instance.
   * @param {string} keypath - Keypath of message resource.
   * @param {Object} options - optional settings.
   * @param {Object} options.vars - Dynamic variables.
   * @returns {?string} - Message
   */
  function l (keypath, options) {
    options = options || {}
    let $vars = l.$vars
    l.$vars = options.vars || {}
    let result = l
    for (let key of String(keypath).split('.')) {
      result = result && result[ key ]
    }
    l.$vars = $vars // Restore $vars
    return result
  }

  // noinspection Eslint
  l.__proto__ = Object.assign(new Function(''), {
    $vars: {},
    toJSON () {
      return Object.assign({}, l)
    }
  })

  let properties = Object.keys(src || {}).reduce((properties, name) => {
    return Object.assign(properties, {
      [name]: {
        get () {
          let value = src[ name ]
          let vars = l.$vars
          let defaults = defineLocale.defaultContext
          return evaljson(value, defaults, src, { l, vars })
        },
        enumerable: true
      }
    })
  }, {})
  Object.defineProperties(l, properties)
  return l
}

Object.assign(defineLocale, {
  defaultContext: Object.assign({}, stringcase, inflection)
})

module.exports = defineLocale
